package GUI;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.RenderingHints;
import java.awt.Shape;
import java.awt.Stroke;
import java.awt.geom.Ellipse2D;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;

public class Drawer extends JPanel {
	//http://stackoverflow.com/questions/8693342/drawing-a-simple-line-graph-in-java
	

	   private static final int PREF_W = 600;
	   private static final int PREF_H = 650;
	   private static final Color GRAPH_COLOR = Color.green;
	   private static final Color COLOR_USER_A = new Color(255, 0, 0, 120);
	   private static final Color COLOR_FALSE_USER = new Color(0, 0, 0, 255);
	   private static final Color COLOR_USER_B = new Color(0, 255, 0, 120);
	   private static final Color COLOR_USER_C = new Color(50, 50, 255, 120);
	   private static final Color COLOR_CORRECT_POINTS = new Color(0, 0, 0, 255);
	   
	   private static final Stroke GRAPH_STROKE = new BasicStroke(3f);
	   private static final int GRAPH_POINT_WIDTH = 8;
	   private List<Integer[]> predicted;
	   private double threshold; //in pixels

	   public Drawer(List<Integer[]> predicted, double threshold) {
	      this.predicted = predicted;
	      this.threshold = threshold;
	   }

	   @Override
	   protected void paintComponent(Graphics g) {
	      super.paintComponent(g);
	      Graphics2D g2 = (Graphics2D)g;
	      g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
	      
	      List<Point> pointsOfUserA = new ArrayList<Point>();
	      List<Point> pointsOfUserB = new ArrayList<Point>();
	      List<Point> pointsOfUserC = new ArrayList<Point>();
	      List<Point> realPoints = new ArrayList<Point>();
	      List<Point> pointsOfFalseUser = new ArrayList<Point>();
	      
	      for (int i = 0; i < predicted.size(); i++) {
	    	 Integer[] point = predicted.get(i);
	    	 
	    	 //System.out.println("point: x: " + point[0] + ", y: " + point[1] + ", z: " + point[2]);
	    	 int x1 = point[0];
	         int y1 = point[1];
	    	 
	    	 if(point[2] == 1) { //UserA
	    		 pointsOfUserA.add(new Point(x1, y1));
	    	 }
	    	 if(point[2] == 2) { //UserB
	    		 pointsOfUserB.add(new Point(x1, y1));
	    	 }
	    	 if(point[2] == 3) { //UserC
	    		 pointsOfUserC.add(new Point(x1, y1));
	    	 }
	    	 if(point[2] == 0) { //Correct points
	    		 realPoints.add(new Point(x1, y1));
	    	 }
	    	 if(point[2] == 4) { //FalseUser
	    		 pointsOfFalseUser.add(new Point(x1, y1));
	    	 }
	      }
	      
	      Stroke oldStroke = g2.getStroke();
	      g2.setColor(GRAPH_COLOR);
	      g2.setStroke(GRAPH_STROKE);
	      /*
	      for (int i = 0; i < graphPoints.size() - 1; i++) {
	         int x1 = graphPoints.get(i).x;
	         int y1 = graphPoints.get(i).y;
	         int x2 = graphPoints.get(i + 1).x;
	         int y2 = graphPoints.get(i + 1).y;
	         g2.drawLine(x1, y1, x2, y2);         
	      }
	       */
	      
	      g2.setStroke(oldStroke);   
	      
	      // draw circles of threshold
	      g2.setColor(COLOR_CORRECT_POINTS);
	      for (int i = 0; i < realPoints.size(); i++) {
	    	  Shape theCircle = new Ellipse2D.Double(realPoints.get(i).x - threshold, realPoints.get(i).y - threshold, 2.0 * threshold, 2.0 * threshold);
	    	  g2.draw(theCircle);
	      }
	      
	      //Draw points of FalseUser
	      g2.setColor(COLOR_FALSE_USER);
	      for (int i = 0; i < pointsOfFalseUser.size(); i++) {
	         int x = pointsOfFalseUser.get(i).x - GRAPH_POINT_WIDTH / 2;
	         int y = pointsOfFalseUser.get(i).y - GRAPH_POINT_WIDTH / 2;
	         //System.out.println("UserA: x = " + x + ", y = " + y);
	         int ovalW = GRAPH_POINT_WIDTH;
	         int ovalH = GRAPH_POINT_WIDTH;
	         //g2.fillOval(x, y, ovalW, ovalH);
	         g2.fillRect(x, y, ovalW, ovalW);
	      }
	      
	      //Draw points of UserA
	      g2.setColor(COLOR_USER_A);
	      for (int i = 0; i < pointsOfUserA.size(); i++) {
	         int x = pointsOfUserA.get(i).x - GRAPH_POINT_WIDTH / 2;
	         int y = pointsOfUserA.get(i).y - GRAPH_POINT_WIDTH / 2;
	         //System.out.println("UserA: x = " + x + ", y = " + y);
	         int ovalW = GRAPH_POINT_WIDTH;
	         int ovalH = GRAPH_POINT_WIDTH;
	         g2.fillOval(x, y, ovalW, ovalH);
	      }
	      
	      //Draw points of UserB
	      g2.setColor(COLOR_USER_B);
	      for (int i = 0; i < pointsOfUserB.size(); i++) {
	         int x = pointsOfUserB.get(i).x - GRAPH_POINT_WIDTH / 2;
	         int y = pointsOfUserB.get(i).y - GRAPH_POINT_WIDTH / 2;
	         //System.out.println("UserB: x = " + x + ", y = " + y);
	         int ovalW = GRAPH_POINT_WIDTH;
	         int ovalH = GRAPH_POINT_WIDTH;
	         g2.fillOval(x, y, ovalW, ovalH);
	      }
	      
	      //Draw points of UserC
	      g2.setColor(COLOR_USER_C);
	      for (int i = 0; i < pointsOfUserC.size(); i++) {
	         int x = pointsOfUserC.get(i).x - GRAPH_POINT_WIDTH / 2;
	         int y = pointsOfUserC.get(i).y - GRAPH_POINT_WIDTH / 2;
	         //System.out.println("UserC: x = " + x + ", y = " + y);
	         int ovalW = GRAPH_POINT_WIDTH;
	         int ovalH = GRAPH_POINT_WIDTH;
	         g2.fillOval(x, y, ovalW, ovalH);
	      }
	      
	    //Draw correct points
	      g2.setColor(COLOR_CORRECT_POINTS);
	      for (int i = 0; i < realPoints.size(); i++) {
	         int x = realPoints.get(i).x - (GRAPH_POINT_WIDTH + 7) / 2;
	         int y = realPoints.get(i).y - (GRAPH_POINT_WIDTH + 7) / 2;
	         //System.out.println("UserC: x = " + x + ", y = " + y);
	         int ovalW = (GRAPH_POINT_WIDTH + 7);
	         int ovalH = (GRAPH_POINT_WIDTH + 7);
	         g2.fillOval(x, y, ovalW, ovalH);
	      }
	   }

	   @Override
	   public Dimension getPreferredSize() {
	      return new Dimension(PREF_W, PREF_H);
	   }

	   

}
