#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
=========================================
Diplomova prace - Biometricka autentizace
Autor: Jan Kozlovsky
=========================================
"""
print(__doc__)

import os
import sys
import math
import csv                                                                          #
import numpy as np                                                                  #
import matplotlib.pyplot as plt                                                     #
from matplotlib.colors import ListedColormap                                        #       Nastaveni aplikace
from sklearn import neighbors, datasets                                             #
from sklearn.naive_bayes import GaussianNB                                          #
from sklearn.naive_bayes import MultinomialNB                                    #  #  #
from sklearn.naive_bayes import BernoulliNB                                       # # #
                                                                                   ###
def convertToFloat(s):                                                              #
    try:
        return float(s)
    except ValueError:
        return s

# POČÁTEČNÍ NASTAVENÍ:========================================================================================================================================================
velikonocnizajic = 0   # pokud velikonocnizajic = 1, je vyhodnocováno heslo velikonocnizajic. Pokud velikonocnizajic = 0, je vyhodnocováno heslo optimalizace
method = 'knn'
                # pokud method = 'knn' je použit algoritmus k-nejbližších sousedů
                # pokud method = 'gausBC' je použit Gausův Bayesovský klasifikátor
                # pokud method = 'multinomicBC' je použit multinomický Bayesovský klasifikátor
                # pokud method = 'bernoulliBC' je použit Bernoulliho Bayesovský klasifikátor
falseUser_threshold = 0.55 # práh, do kterého je ještě akceptován uživatel jako správný. U algoritmu K-NN tato proměnná představuje vzdálenost predikovaného výsledku od vzorového
#============================================================================================================================================================================

# Nastavení cest ke zdrojovým souborům
dir = os.path.dirname(__file__)

if velikonocnizajic == 1:
    numOfInputs = 32
    learningPasswordsPath   = os.path.join(dir, '_DATA/velikonocnizajic/Python_OutputOfNormalization.csv')
    learningClassesPath     = os.path.join(dir, '_DATA/velikonocnizajic/InputForNormalization.csv')

    testingPasswordsPath    = os.path.join(dir, '_DATA/velikonocnizajic/testing/Python_OutputOfNormalization.csv')
    testingClassesPath      = os.path.join(dir, '_DATA/velikonocnizajic/testing/Python_InputForNormalization.csv')
else:
    numOfInputs = 24
    learningPasswordsPath   = os.path.join(dir, '_DATA/optimalizace/Python_OutputOfNormalization.csv') 
    learningClassesPath     = os.path.join(dir, '_DATA/optimalizace/InputForNormalization.csv')

    testingPasswordsPath    = os.path.join(dir, '_DATA/optimalizace/testing/Python_OutputOfNormalization.csv')
    testingClassesPath      = os.path.join(dir, '_DATA/optimalizace/testing/Python_InputForNormalization.csv')


# Přečtení souborů a určení počtu hesel pro nastavení velikost polí
numOfLearningPasswords = -1
with open(learningPasswordsPath, 'rb') as csvfile:
    reader = csv.reader(csvfile, delimiter=',', quotechar='"')
    for row in reader:
        numOfLearningPasswords = numOfLearningPasswords + 1

numOfTestingPasswords = -1
with open(testingPasswordsPath, 'rb') as csvfile:
    reader = csv.reader(csvfile, delimiter=',', quotechar='"')
    for row in reader:
        numOfTestingPasswords = numOfTestingPasswords + 1

listOfPsw = [[0 for x in range(numOfInputs)] for x in range(numOfLearningPasswords)]
listOfRes = [0 for x in range(numOfLearningPasswords)]

listOfTestPsw = [[0 for x in range(numOfInputs)] for x in range(numOfTestingPasswords)]
listOfTestRes = [0 for x in range(numOfTestingPasswords)]

# Načtení hesel pro naučení
print 'Nacitani vzorovych hesel'
numOfRows = 0
didWePassHeadersAlready = 0
with open(learningPasswordsPath, 'rb') as csvfile:
    reader = csv.reader(csvfile, delimiter=',', quotechar='"')
    for row in reader:
        numOfCells = 0
        if didWePassHeadersAlready == 1:
            for cell in row:
                y = convertToFloat(cell)
                if numOfCells <= numOfInputs - 1:
                    listOfPsw[numOfRows][numOfCells] = y
                                        
                numOfCells = numOfCells + 1

            numOfRows = numOfRows + 1
            
        didWePassHeadersAlready = 1

# Načtení výstupů pro naučení
print 'Nacitani vzorovych trid'      
numOfRows = 0
didWePassHeadersAlready = 0
with open(learningClassesPath, 'rb') as csvfile:
    reader = csv.reader(csvfile, delimiter=',', quotechar='"')
    for row in reader:
        numOfCells = 0
        if didWePassHeadersAlready == 1:
            for cell in row:
                y = convertToFloat(cell)
                if numOfCells == numOfInputs:
                    listOfRes[numOfRows] = y
                    
                numOfCells = numOfCells + 1

            numOfRows = numOfRows + 1
            
        didWePassHeadersAlready = 1

# Načtení hesel pro testování
print 'Nacitani testovacich hesel'
numOfRows = 0
didWePassHeadersAlready = 0
with open(testingPasswordsPath, 'rb') as csvfile:
    reader = csv.reader(csvfile, delimiter=',', quotechar='"')
    for row in reader:
        numOfCells = 0
        if didWePassHeadersAlready == 1:
            for cell in row:
                y = convertToFloat(cell)
                if numOfCells <= numOfInputs - 1:
                    listOfTestPsw[numOfRows][numOfCells] = y
                    
                numOfCells = numOfCells + 1

            numOfRows = numOfRows + 1
            
        didWePassHeadersAlready = 1

# Načtení výstupů pro testování
print 'Nacitani testovacich trid'
numOfRows = 0
didWePassHeadersAlready = 0
with open(testingClassesPath, 'rb') as csvfile:
    reader = csv.reader(csvfile, delimiter=',', quotechar='"')
    for row in reader:
        numOfCells = 0
        if didWePassHeadersAlready == 1:
            for cell in row:
                y = convertToFloat(cell)
                if numOfCells == numOfInputs:
                    listOfTestRes[numOfRows] = y
                    
                numOfCells = numOfCells + 1

            numOfRows = numOfRows + 1
            
        didWePassHeadersAlready = 1

# Vyhodnocení
# http://scikit-learn.org/stable/modules/naive_bayes.html
# http://scikit-learn.org/stable/modules/generated/sklearn.neighbors.KNeighborsClassifier.html#sklearn.neighbors.KNeighborsClassifier 
print '========================================================'
print 'Vysledky:'
h = .02  # step size in the mesh
alpha = 1.6

if method == 'knn':
    clf = neighbors.KNeighborsClassifier(n_neighbors=1, weights='uniform', algorithm='auto', leaf_size=30, p=2, metric='minkowski' )
if method == 'gausBC':
    clf = GaussianNB()
if method == 'multinomicBC':
    clf = MultinomialNB(alpha)
if method == 'bernoulliBC':
    clf = BernoulliNB()

clf.fit(listOfPsw, listOfRes)

isResultAccepted = 2
iterator = 0
numOfErrors = 0
numOfFalseUsers = 0
numOfTooFarButCorrect = 0
numOfAcceptedButFalse = 0
maxDistance = 0
minDistance = 99999
distance = 0

for psw in listOfTestPsw:
    Predicted = clf.predict(psw)
    Ideal = listOfTestRes[iterator]
    Predicted_proba = clf.predict_proba(psw)

    if Ideal == 'FalseUser': numOfFalseUsers = numOfFalseUsers + 1

    if Ideal == 'Honza': Ideal = 'User_A'
    if Ideal == 'Anicka': Ideal = 'User_B'
    if Ideal == 'Marketka': Ideal = 'User_C'

    if Predicted == 'Honza': Predicted = 'User_A'
    if Predicted == 'Anicka': Predicted = 'User_B'
    if Predicted == 'Marketka': Predicted = 'User_C'

    if method == 'knn':  # pouze algoritmus k-NN poskytuje informaci o vzdalenosti, dle které je možno určit věrohodnost výstupu.
                         # Bayesovské klasifikatory tuto moznost nemaji.
        distance = clf.kneighbors([psw])[0][0]
        
        if distance > falseUser_threshold:
            isResultAccepted = 0
        else:
            isResultAccepted = 1             
        if distance > maxDistance: maxDistance = distance
        if distance < minDistance: minDistance = distance
    
    if Predicted == Ideal:
        print str(iterator + 1) + ': Predikovano: ' + str(Predicted) + ', P = ' + str(Predicted_proba) + ', Spravne: ' + str(Ideal) + '   SPRAVNA PREDIKCE'
        if isResultAccepted == 0:
            numOfTooFarButCorrect = numOfTooFarButCorrect + 1
            print '...ALE ODMITNUTO JAKO MALO VEROHODNY VYSLEDEK - Chybne odmitnut spravny uzivatel'
        if isResultAccepted == 1:
            print 'PRIJATO JAKO VEROHODNY VYSLEDEK - Spravne prijat spravny uzivatel'
    else:
        print str(iterator + 1) + ': Predikovano: ' + str(Predicted) + ', P = ' + str(Predicted_proba) + ', Spravne: ' + str(Ideal) + '   CHYBNA PREDIKCE!!!'
        if (Ideal != 'FalseUser'): numOfErrors = numOfErrors + 1
        if isResultAccepted == 0:
            print 'ODMITNUTO JAKO MALO VEROHODNY VYSLEDEK - Spravne odmitnut cizi uzivatel'
        if isResultAccepted == 1:
            numOfAcceptedButFalse = numOfAcceptedButFalse + 1
            print 'PRIJATO JAKO VEROHODNY VYSLEDEK - Chybne prijat cizi uzivatel'

    iterator = iterator + 1
    if method == 'knn':
        print 'Vzdalenost: ' + str(distance)

    print ' '

print 'Pocet testovacich vzorku: ' + str(iterator)
print 'Pocet cizich uzivatelu: ' + str(numOfFalseUsers)
print 'Pocet chybne predikovanych uzivatelu vyjma cizich uzivatelu (napr. predikovan byl User_A, pricemz jde ve skutecnosti o User_B): ' + str(numOfErrors)
if method == 'knn':
    print 'Max vzdalenost: ' + str(maxDistance)
    print 'Min vzdalenost: ' + str(minDistance)
    print 'Pocet chybne akceptovanych cizich uzivatelu: ' + str(numOfAcceptedButFalse) + ' / ' + str(numOfFalseUsers) 
    print 'Pocet chybne odmitnutych spravnych uzivatelu: ' + str(numOfTooFarButCorrect) + ' / ' + str(iterator - numOfFalseUsers)

